import * as mc from "@minecraft/server";

let dimensions = ["overworld", "nether", "the_end"];

const afterEvent = mc.world.afterEvents;

afterEvent.itemUse.subscribe((data) => {
    let player = data.source;
    let item = data.itemStack;
    if (item.typeId == "brs:magnet") {
        if (!player.getTags().includes("magnet_on")) {
            player.addTag("magnet_on");
        } else {
            player.removeTag("magnet_on");
        }
    }
});

mc.system.runInterval(() => {
    dimensions.forEach((dimension) => {
        const thisDimension = mc.world.getDimension(dimension);
        const players = thisDimension && thisDimension.getPlayers();
        if (!thisDimension || !players) {
            return
        }
        players.forEach((player) => {
            const items = thisDimension.getEntities({
                location: player.location,
                maxDistance: 10,
                type: "minecraft:item",
            });
            if (!items) {
                return
            };
            items.forEach((item) => {
                const impulse = calculateImpulse(player.location, item.location, 0.1);
                if (player.hasTag("magnet_on")) {
                    item.applyImpulse(impulse);
                }
            })

        })
    });
})

function calculateImpulse(player_location, item_location, speed) {
    // Calculate the direction vector from item2 to item1
    let direction = {
        x: player_location.x - item_location.x,
        y: player_location.y - item_location.y,
        z: player_location.z - item_location.z,
    };

    // Normalize the direction vector (to get a unit vector)
    let magnitude = Math.sqrt(direction.x * direction.x + direction.y * direction.y + direction.z * direction.z);

    let unitDirection = {
        x: direction.x / magnitude,
        y: direction.y / magnitude,
        z: direction.z / magnitude,
    };

    // Calculate the impulse needed to move at 'speed' towards item1
    let impulse = {
        x: unitDirection.x * speed,
        y: unitDirection.y * speed,
        z: unitDirection.z * speed,
    };

    return impulse;
}